@@ -2,9 +2,7 @@
 
 #include <ctype.h>
 #include <dlfcn.h>
-#include <errno.h>
 #include <fcntl.h>
-#include <stdarg.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -14,7 +12,6 @@
 
 #include "dynamic_filters.h"
 #include "file.h"
-#include "image.h"
 #include "script.h"
 
 static ImlibExternalFilter *filters = NULL;
@@ -98,7 +95,7 @@ __imlib_dynamic_filters_init()
 #ifdef FDEBUG
         printf("DEBUG: Loading Filters\n");
 #endif
-        list = __imlib_ListModules(__imlib_PathToFilters(), &num_filters);
+        list = __imlib_ModulesList(__imlib_PathToFilters(), &num_filters);
         for (i = num_filters - 1; i >= 0; i--)
           {
              tptr = NULL;
