@@ -11,12 +11,13 @@
  * Version: 2004-08-28
  *------------------------------------------------------------------------------*/
 
-#include "loader_common.h"
-
-#include <sys/mman.h>
+#include "config.h"
+#include "Imlib2_Loader.h"
 
 #define DBG_PFX "LDR-lbm"
 
+static const char  *const _formats[] = { "iff", "ilbm", "lbm" };
+
 #define L2RLONG(a) ((((int)((a)[0]) & 0xff) << 24) + (((int)((a)[1]) & 0xff) << 16) + (((int)((a)[2]) & 0xff) << 8) + ((int)((a)[3]) & 0xff))
 #define L2RWORD(a) ((((int)((a)[0]) & 0xff) << 8) + ((int)((a)[1]) & 0xff))
 
@@ -276,11 +277,11 @@ scalecmap(ILBM * ilbm)
 }
 
 /*------------------------------------------------------------------------------
- * Deplanes and converts an array of bitplanes to a single scanline of DATA32
- * (unsigned int) values. DATA32 is ARGB.
+ * Deplanes and converts an array of bitplanes to a single scanline of uint32_t
+ * (unsigned int) values. uint32_t is ARGB.
  *------------------------------------------------------------------------------*/
 static void
-deplane(DATA32 * row, int w, ILBM * ilbm, unsigned char *plane[])
+deplane(uint32_t * row, int w, ILBM * ilbm, unsigned char *plane[])
 {
    unsigned int        l, r, g, b, a;
    int                 i, o, x;
@@ -438,11 +439,10 @@ deplane(DATA32 * row, int w, ILBM * ilbm, unsigned char *plane[])
  *
  * Imlib2 doesn't support reading comment chunks like ANNO.
  *------------------------------------------------------------------------------*/
-int
-load2(ImlibImage * im, int load_data)
+static int
+_load(ImlibImage * im, int load_data)
 {
    int                 rc;
-   void               *fdata;
    char               *env;
    int                 i, n, y, z;
    unsigned char      *plane[40];
@@ -450,10 +450,6 @@ load2(ImlibImage * im, int load_data)
 
    rc = LOAD_FAIL;
 
-   fdata = mmap(NULL, im->fsize, PROT_READ, MAP_SHARED, fileno(im->fp), 0);
-   if (fdata == MAP_FAILED)
-      return LOAD_BADFILE;
-
    plane[0] = NULL;
    memset(&ilbm, 0, sizeof(ilbm));
 
@@ -461,7 +457,7 @@ load2(ImlibImage * im, int load_data)
    * Load the chunk(s) we're interested in. If load_data is not true, then we only
    * want the image size and format.
    *----------*/
-   if (!loadchunks(fdata, im->fsize, &ilbm, load_data))
+   if (!loadchunks(im->fi->fdata, im->fi->fsize, &ilbm, load_data))
       goto quit;
 
    rc = LOAD_BADIMAGE;          /* Format accepted */
@@ -489,13 +485,13 @@ load2(ImlibImage * im, int load_data)
 
    ilbm.mask = ilbm.bmhd.data[9];
 
-   UPDATE_FLAG(im->flags, F_HAS_ALPHA, ilbm.mask || ilbm.depth == 32);
+   im->has_alpha = ilbm.mask != 0 || ilbm.depth == 32;
 
    env = getenv("IMLIB2_LBM_NOMASK");
    if (env
        && (!strcmp(env, "true") || !strcmp(env, "1") || !strcmp(env, "yes")
            || !strcmp(env, "on")))
-      UNSET_FLAG(im->flags, F_HAS_ALPHA);
+      im->has_alpha = 0;
 
    if (!load_data)
       QUIT_WITH_RC(LOAD_SUCCESS);
@@ -563,33 +559,11 @@ load2(ImlibImage * im, int load_data)
    * We either had a successful decode, the user cancelled, or we couldn't get
    * the memory for im->data or plane[0].
    *----------*/
-   if (rc <= 0)
-      __imlib_FreeData(im);
-
    free(plane[0]);
 
    freeilbm(&ilbm);
 
-   munmap(fdata, im->fsize);
-
    return rc;
 }
 
-/*------------------------------------------------------------------------------
- * Perhaps save only in 32-bit format? The IFF ILBM format has pretty much gone
- * the way of the Amiga, who saves in this format any more?
- *------------------------------------------------------------------------------*/
-#if 0
-char
-save(ImlibImage * im, ImlibProgressFunction progress, char progress_granularity)
-{
-   return 0;
-}
-#endif
-
-void
-formats(ImlibLoader * l)
-{
-   static const char  *const list_formats[] = { "iff", "ilbm", "lbm" };
-   __imlib_LoaderSetFormats(l, list_formats, ARRAY_SIZE(list_formats));
-}
+IMLIB_LOADER(_formats, _load, NULL);
